package com.jwipc.supplier.client.convert;

import com.jwipc.supplier.clent.dto.*;
import com.jwipc.supplier.entity.*;
import org.mapstruct.Mapper;

import java.util.List;


@Mapper(componentModel = "spring")
public interface SupplierClientConvertConvert {
    List<SupplierDTO> entity2DTOs(List<Supplier> list);

    SupplierPurchaseCategoryDTO entity2Dto(SupplierPurchaseCategory purchaseCategory);

    SupplierDTO entity2DTO(Supplier list);

    List<SupplierBankDTO> entity2DTO(List<SupplierBank> supplierBank);

    SupplierContactsDTO contactEntity2DTO(SupplierContacts supplierContacts);

    SupplierAvlBaseDTO avlToDto(SupplierAvl supplierAvl);

    List<SupplierContactsDTO> contactEntities2DTOs(List<SupplierContacts> supplierContacts);

    SupplierTemplateDTO convert2(SupplierTemplate one);

    List<SupplierPurchaseCategoryDTO> dto2Entities(List<SupplierPurchaseCategory> list);

    CustomerInfoClientDTO customerInfoEntity2ClientDTO(CustomerInfo customerInfo);

    SupplierInvitaionDTO coEntity2DTO(SupplierInvitaion supplierInvitaion);
}
